#include "stm32f10x.h"

//********************************* Platform dependent functions ********************************* 
// Modify these functions for usage with other microcontrollers.
// This version is prepared to work with display connected to SPI1.
// 
// Display to microcontroller connections are:
//   Display      STM32
//   /LOAD        PA8
//   SDCLK        PA5
//   DATA         PA7
//   /RST         PA4
//   nLOAD1       PA8
//   nLOAD2       PA9

void SS_Low()   { GPIOA->ODR &= ~(1 << 4); }
void SS_High()  { GPIOA->ODR |= (1 << 4); }

void RES_Low()   { GPIOA->ODR &= ~(1 << 0); }
void RES_High()  { GPIOA->ODR |= (1 << 0); }

void DC_Low()   { GPIOA->ODR &= ~(1 << 1); }
void DC_High()  { GPIOA->ODR |= (1 << 1); }




//***************************************************************************************************************
// Function name: SPI_Transmit
//    Serial data transmission.
//
// Parameters:
//    cData - byte to send
//
// Return value
//    None
//
//***************************************************************************************************************
char SPI_Transmit (char outb) 
{
  	SPI1->DR = outb;	 

  	while (!(SPI1->SR & SPI_SR_RXNE));
 
  	return SPI1->DR;  
}


//***************************************************************************************************************
// Function name: InitDisplay 
//    Initialization of SPI (only SCK and MOSI are needed) and GPIO lines for display /RST and /LOAD lines.
//
// Parameters:
//    None
//
// Return value
//    None
//
//***************************************************************************************************************
void ISC15ANP4_Init(void)
{
	int i;
	
	// Enable clock for GPIOA, SPI1
	RCC->APB2ENR |= RCC_APB2ENR_SPI1EN | RCC_APB2ENR_IOPAEN | RCC_APB2ENR_AFIOEN;

	// Configure GPIOA lines
	//  - pins 5, 7 as Alternate-function push-pull output (50 MHz), pin 4 as general purpose output (50 MHz) 
	GPIOA->CRL = GPIOA->CRL & ~(GPIO_CRL_MODE4 | GPIO_CRL_MODE5 | GPIO_CRL_MODE7 | GPIO_CRL_CNF4 | GPIO_CRL_CNF5 | GPIO_CRL_CNF7);
	GPIOA->CRL |= GPIO_CRL_MODE4_0 | GPIO_CRL_MODE4_1 | GPIO_CRL_MODE5_0 | GPIO_CRL_MODE5_1 | GPIO_CRL_CNF5_1 | 
	              GPIO_CRL_MODE7_0 | GPIO_CRL_MODE7_1 | GPIO_CRL_CNF7_1;
	
	// Configure SPI1
	//   Master, LSB first, 8-bit frame, fPCLK/16, CLK high on idle, second edge
	//   For 72 MHz SYSCLK SPI frequency is 4.5 MHz
	SPI1->CR1 = SPI_CR1_BR_0 | SPI_CR1_BR_1 | SPI_CR1_MSTR | SPI_CR1_CPOL | SPI_CR1_CPHA;
	SPI1->CR2 = SPI_CR2_SSOE;

	SPI1->CR1 |= SPI_CR1_SPE;


	// Configure PA0, PA1 as general purpose output (50 MHz)
	GPIOA->CRL = GPIOA->CRL & ~(GPIO_CRL_MODE0 | GPIO_CRL_CNF0 | GPIO_CRL_MODE1 | GPIO_CRL_CNF1);
	GPIOA->CRL |= GPIO_CRL_MODE0_0 | GPIO_CRL_MODE0_1 | GPIO_CRL_MODE1_0 | GPIO_CRL_MODE1_1;
	RES_High();
	DC_High();

	// The Reset pin should be set to low for 3 us and then set to high
	for (i=0; i<0xF; i++);
	RES_Low();
	for (i=0; i<0xF; i++);
	RES_High();


	DC_Low();
	
	SS_Low();
	SPI_Transmit(0x81); SPI_Transmit(0x19); // Contras for color A
	SS_High();

	SS_Low();
	SPI_Transmit(0x82); SPI_Transmit(0x14);	// Contras for color B
	SS_High();

	SS_Low();
	SPI_Transmit(0x83); SPI_Transmit(0x24);	// Contras for color C
	SS_High();

	SS_Low();
	SPI_Transmit(0x87); SPI_Transmit(0x0F);	// Master current control
	SS_High();

	SS_Low();
	SPI_Transmit(0xA0); SPI_Transmit(0x70);	// Remap & color depth setting
	SS_High();

	SS_Low();
	SPI_Transmit(0xA1); SPI_Transmit(0x00);	// Set display start line
	SS_High();

	SS_Low();
	SPI_Transmit(0xA2); SPI_Transmit(0x10);	// Set display offset
	SS_High();

	SS_Low();
	SPI_Transmit(0xA4); 					// Normal display
	SS_High();

	SS_Low();
	SPI_Transmit(0xA8); SPI_Transmit(0x2F);	// Multiplex ratio
	SS_High();

	SS_Low();
	SPI_Transmit(0xAB); SPI_Transmit(0x00); SPI_Transmit(0x12); SPI_Transmit(0x0C); SPI_Transmit(0x14); SPI_Transmit(0x12); //Dim mode setting for color A, B &C
	SS_High();

	SS_Low();
	SPI_Transmit(0xAD); SPI_Transmit(0x8E);	// Master configuration
	SS_High();

	SS_Low();
	SPI_Transmit(0xB0); SPI_Transmit(0x0B);	// Power save mode
	SS_High();

	SS_Low();
	SPI_Transmit(0xB1); SPI_Transmit(0x44);	// Phase 1 and 2 period adjustment
	SS_High();

	SS_Low();
	SPI_Transmit(0xB3); SPI_Transmit(0xA0);	// Display clock divider/ oscillator frequency
	SS_High();

	SS_Low();
	SPI_Transmit(0xB9);						// Enable linear gray scale
	SS_High();

	SS_Low();
	SPI_Transmit(0xBB); SPI_Transmit(0x12);	// Pre charge level
	SS_High();

	SS_Low();
	SPI_Transmit(0xBE); SPI_Transmit(0x28);	// Set Vcomh
	SS_High();

	SS_Low();
	SPI_Transmit(0xAF); 					// Display on in normal mode
	SS_High();

	DC_High();
}

void ISC15ANP4_SetRangeToFullScreen()
{
	DC_Low();

	SS_Low();
	SPI_Transmit(0x15); SPI_Transmit(0x10); SPI_Transmit(0x4F);	// Set column range
	SS_High();

	SS_Low();
	SPI_Transmit(0x75); SPI_Transmit(0x0); SPI_Transmit(0x2F);	// Set row range
	SS_High();

	DC_High();	
}

void ISC15ANP4_DrawScreen(char * buffer)
{
	int i;
	DC_High();
	for (i = 0; i < 64*48*2; i++)
	{
		SS_Low();
		SPI_Transmit(buffer[i]);
		SS_High();
	}
}

void ISC15ANP4_Transition(char * from, char * to)
{
	int x, y, step = 0;
	DC_High();
	for (step = 0; step < 64; step+=5)
	{
		for (y = 0; y < 48; y++)
		{
			for (x = 0; x <= step; x++)
			{
				SS_Low();
				SPI_Transmit(to[2*(y * 64 + (63-step+x))]);
				SS_High();	
				SS_Low();
				SPI_Transmit(to[2*(y * 64 + (63-step+x))+1]);
				SS_High();	
			}
			for (x = step + 1; x < 64; x++)
			{
				SS_Low();
				SPI_Transmit(from[2*(y*64 + x -step)]);
				SPI_Transmit(from[2*(y*64 + x -step)+1]);
				SS_High();	
			}
		}
	}
	ISC15ANP4_DrawScreen(to);
}






